function [w, ep] = minlipnew(model,Y,delta)
% Implementation of minlip using Yalmip
% Use censored failure time data
% event(not censored): delta=1;  censored: delta=0
%
% >> w=minlip(model)
% >> w=minlip(X,Y,delta)
% >> [w,ep]=minlip(model)
% >> [w,ep]=minlip(X,Y,delta)
%
% Input:
% Case 1: Structure format:
% 'model' is a structure which includes the fields 'func','X','Y','delta','gam','mu'
% Case 2: Original format:
% 'X' input: is a matrix of n x d
% 'Y' time: is a vector of n elements
% 'delta' censoring: is a vector of n elements (0-1 values, 0 means events and 1 means censored)
% 'gam' a parameter: is a scalar
%
% Output:
% 'w' parameter: is a vector of d elements
% 'ep' residuals: is a vector of n elements
%
% Author: Liu,Yang (liu.yang@it.uu.se)

% Selection of input format
if nargin==1
    [~,X,Y,delta]=ungroup(model);
else
    X=model;
end

[n,d]=size(X);

% Sort by Y
[Y,events] = sort(Y);
X= X(events,:);
delta=delta(events,:);

% Build matrix D and Dc
D1=speye(n);
D2=speye(n);
D=D1(2:n,:)-D2(1:n-1,:);
I=(D*Y>0 & delta(1:n-1)==1);
R=diag(I);
Dc=R*D;
Dc=sparse(Dc);

% Solve QP by solver
yalmip('clear');
w=sdpvar(d,1);
ep=sdpvar(n-1,1);
solvesdp([Dc*(X*w)>=diag(I.*ep)*D*Y,norm(w,2)<=1],...
    -1*ep,...
    sdpsettings('solver','mosek','verbose',0,'cachesolvers',1));

% Convert the result to double type
w=double(w);
ep=double(ep);