function [w, ep] = model2new(model,Y,delta,gam,mu)
% Implementation of model2new using Yalmip
% Use censored failure time data
% event(not censored): delta=1;  censored: delta=0
%
% >> w=model2new(model)
% >> w=model2new(X,Y,delta,gam,mu)
% >> [w,ep]=model2new(model)
% >> [w,ep]=model2new(X,Y,delta,gam,mu)
%
% Input:
% Case 1: Structure format:
% 'model' is a structure which includes the fields 'func','X','Y','delta','gam','mu'
% Case 2: Original format:
% 'X' input: is a matrix of n x d
% 'Y' time: is a vector of n elements
% 'delta' censoring: is a vector of n elements (0-1 values, 0 means events and 1 means censored)
% 'gam' a parameter: is a scalar
% 'mu' a parameter: is a scalar
%
% Output:
% 'w' parameter: is a vector of d elements
% 'ep' residuals: is a vector of n elements
%
% Author: Liu,Yang (liu.yang@it.uu.se)

% Selection of input format
if nargin==1
    [~,X,Y,delta,gam,mu]=unfold(model);
else
    X=model;
end

% Sort by Y
[n,d]=size(X);
[Y,events] = sort(Y);
X= X(events,:);
delta=delta(events,:);

% Build matrix D and Dc
Dc=buildDMatrix(n,delta,Y);

% Solve QP by solver
yalmip('clear');
w=sdpvar(d,1);
ep=sdpvar(n-1,1);
sig1=sdpvar(n,1);
sig2=sdpvar(n,1);
b=sdpvar(n,1);
solvesdp([Dc*(X*w)+ep>=Dc*Y, ...
    X*w+b>=Y-sig1, ...
    -delta.*(X*w+b)>=-delta.*Y-sig2, ...
    ep>=0,...
    sig1>=0,...
    sig2>=0],...
    norm(w,2)+gam*sum(ep)+mu*sum(sig1+sig2),...
    sdpsettings('solver','mosek','verbose',0,'cachesolvers',1));

% Convert the result to double type
w=double(w);
ep=double(ep);